using System;
using System.Collections;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using CommerceBuilder.Common;
using CommerceBuilder.Marketing;
using CommerceBuilder.Orders;
using CommerceBuilder.Payments;
using CommerceBuilder.Products;
using CommerceBuilder.Shipping;
using CommerceBuilder.Users;
using CommerceBuilder.Utility;

public partial class Admin_Orders_PlaceOrder2 : CommerceBuilder.Web.UI.AbleCommerceAdminPage
{
    int _BasketId;
    Basket _Basket;

    private bool _ShipMethodError = false;

    private int _HasShippableItems = -1;

    private bool HasShippableItems
    {
        get
        {
            if (_HasShippableItems < 0)
            {
                _HasShippableItems = 0;
                foreach (BasketItem item in _Basket.Items)
                {
                    if ((item.OrderItemType == OrderItemType.Product) && (item.Shippable != Shippable.No))
                    {
                        _HasShippableItems = 1;
                        break;
                    }
                }
            }
            return (_HasShippableItems == 1);
        }
        set { _HasShippableItems = (value ? 1 : 0); }
    }

    protected bool HasMultipleShipments()
    {
        return _Basket.Shipments.Count > 1;
    }

    private void ResetSelectedShippingMethods()
    {
        foreach (BasketShipment shipment in _Basket.Shipments)
        {
            shipment.ShipMethodId = 0;
            shipment.Save();
        }
    }

    protected void Page_Init(object sender, EventArgs e)
    {
        //VIEWSTATE IS DISABLED, SO THIS DATA MUST BE BOUND ON EACH VISIT
        //BIND THE COUNTRY CODE FIELD
        BillToCountryCode.DataSource = Token.Instance.Store.Countries;
        BillToCountryCode.DataBind();
        ShipToCountryCode.DataSource = Token.Instance.Store.Countries;
        ShipToCountryCode.DataBind();
        // SELECT DEFAULT WAREHOUSE COUNTRY AS SELECTED
        ListItem selectedCountry = BillToCountryCode.Items.FindByValue(Token.Instance.Store.DefaultWarehouse.CountryCode);
        if (selectedCountry != null) BillToCountryCode.SelectedIndex = BillToCountryCode.Items.IndexOf(selectedCountry);
        selectedCountry = ShipToCountryCode.Items.FindByValue(Token.Instance.Store.DefaultWarehouse.CountryCode);
        if (selectedCountry != null) ShipToCountryCode.SelectedIndex = ShipToCountryCode.Items.IndexOf(selectedCountry);

        // INITIALIZE BASKET
        _BasketId = AlwaysConvert.ToInt(Request.QueryString["BasketId"]);
        _Basket = BasketDataSource.Load(_BasketId);
        if (_Basket == null) Response.Redirect("Default.aspx");
        //RESET SHIPPING CHARGES ON FIRST VISIT
        if (!Page.IsPostBack) ResetSelectedShippingMethods();
        _Basket.Recalculate();
        //BIND THE BASKET ON EVERY VISIT (NO VIEWSTATE)
        BasketSummaryGrid.DataSource = _Basket.Items;
        BasketSummaryGrid.DataBind();
        BasketSummaryGrid.Columns[0].Visible = HasMultipleShipments();
        EditBasketButton.NavigateUrl += _Basket.UserId;
        ShipmentList.DataSource = _Basket.Shipments;
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            // INITIALIZE THE BILLING ADDRESS
            if (_Basket.User != null && !_Basket.User.IsAnonymous)
            {
                BindBillingAddress();
                BindShippingAddress();
            }
            UpdateShipMethod();
        }
        else if (Request.Form["__EVENTTARGET"].EndsWith("ShipMethodList"))
        {
            ShipmentList.DataBind();
            UpdateShipMethod();
        }

        trShippingMethod.Visible = this.HasShippableItems;
        if (!this.HasShippableItems) PaymentPanel.Enabled = true;
    }

    private void BindBillingAddress()
    {
        if (_Basket.User != null)
        {
            CustomerEmail.Text = _Basket.User.Email;
            BillToAddress1.Text = _Basket.User.PrimaryAddress.Address1;
            BillToAddress2.Text = _Basket.User.PrimaryAddress.Address2;
            BillToCity.Text = _Basket.User.PrimaryAddress.City;
            BillToCompany.Text = _Basket.User.PrimaryAddress.Company;

            BillToCountryCode.DataSource = Token.Instance.Store.Countries;
            BillToCountryCode.DataBind();
            ListItem selectedCountry = BillToCountryCode.Items.FindByValue(_Basket.User.PrimaryAddress.CountryCode.ToString());
            if (selectedCountry != null) BillToCountryCode.SelectedIndex = BillToCountryCode.Items.IndexOf(selectedCountry);

            BillToFirstName.Text = _Basket.User.PrimaryAddress.FirstName;
            BillToLastName.Text = _Basket.User.PrimaryAddress.LastName;
            BillToPhone.Text = _Basket.User.PrimaryAddress.Phone;
            BillToPostalCode.Text = _Basket.User.PrimaryAddress.PostalCode;
            BillToProvince.Text = _Basket.User.PrimaryAddress.Province;
        }
    }

    private void BindShippingAddress()
    {
        ShipToCountryCode.DataSource = Token.Instance.Store.Countries;
        ShipToCountryCode.DataBind();
    }

    // BASKET DETAILS
    protected void BasketSummaryGrid_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.Footer)
        {
            Label ProductTotal = (Label)e.Row.FindControl("ProductTotal");
            if (ProductTotal != null)
            {
                ProductTotal.Text = string.Format("{0:lc}", _Basket.Items.TotalPrice());
            }
        }
    }

    protected void ShipToBillingAddress_CheckedChanged(object sender, EventArgs e)
    {
        if (ShipToBillingAddress.Checked)
        {
            ShippingAddressPanel.Enabled = false;
        }
        else
        {
            ShippingAddressPanel.Enabled = true;
        }
        UpdateShipMethod();
    }

    protected void ShipMethodCheckButton_Click(object sender, EventArgs e)
    {
        UpdateShipMethod();
    }

    private void UpdateShipMethod()
    {
        CouponsPanel.Enabled = false;

        // VALIDATE BILLING AND SHIPPING ADDRESSES
        if (Page.IsPostBack && !validateAndCreateAddresses())
        {
            ShipMethodErrorLabel.Text = "Valid Shipping Address is Required";
            Page.Validate("BillingAddress");
            if (!ShipToBillingAddress.Checked) Page.Validate("ShippingAddress");
            return;
        }

        if (!Page.IsPostBack)
        {
            // JUST UPDATE THE SHIPPING METHOD SELECTION IN THE LIST
            for (int i = 0; i < ShipmentList.Items.Count; i++)
            {
                RepeaterItem item = ShipmentList.Items[i];
                if (item != null)
                {
                    int shipMethodId = _Basket.Shipments[i].ShipMethodId;
                    DropDownList ShipMethodList = (DropDownList)item.FindControl("ShipMethodList");
                    if (shipMethodId != 0 && ShipMethodList != null)
                    {
                        ListItem selected = ShipMethodList.Items.FindByValue(shipMethodId.ToString());
                        if (selected != null) selected.Selected = true;
                    }
                }
            }
            return;
        }

        _Basket.Package();
        _Basket.Recalculate();
        ShipmentList.DataBind();

        if (_Basket.Shipments != null && _Basket.Shipments.Count > 0)
        {
            // THERE WILL BE ONLY ONE SHIPMENT
            Address ShipAddress = AddressDataSource.Load(AlwaysConvert.ToInt(Session["CC0_SHIPTOADDRESSID"]));

            // UPDATE SHIPMENTS, ONLY ONE SHIPMENT WILL BE HANDLED
            foreach (BasketShipment shipment in _Basket.Shipments)
            {
                int sessionShipId = AlwaysConvert.ToInt(Session["CC0_SHIPTOADDRESSID"]);
                shipment.AddressId = sessionShipId;
            }

            ShipmentList.DataBind();

            // UPDATE SHIPMENTS 
            for (int i = 0; i < ShipmentList.Items.Count; i++)
            {
                RepeaterItem item = ShipmentList.Items[i];
                if (item != null)
                {
                    BasketShipment shipment = _Basket.Shipments[i];
                    DropDownList ShipMethodList = (DropDownList)item.FindControl("ShipMethodList");
                    if (shipment != null && ShipMethodList != null)
                    {
                        shipment.ShipMethodId = AlwaysConvert.ToInt(ShipMethodList.Items[ShipMethodList.SelectedIndex].Value);
                    }
                }
            }

            _Basket.Shipments.Save();
            _Basket.Save();
        }

        CouponsPanel.Enabled = true;

        // UPDATE THE ORDER ITMES PANEL TO REFLECT ANY CHANGE
        _Basket.Recalculate();
        BasketSummaryGrid.DataBind();
        BasketAjax.Update();

        PaymentPanel.Enabled = !_ShipMethodError;

    }
    private void BindShipMethodRateQuotes(DropDownList ShipMethodList, ICollection<ShipRateQuote> rateQuotes)
    {
        ShipMethodList.Items.Clear();
        foreach (ShipRateQuote quote in rateQuotes)
        {
            String text = quote.Name + " ({0:lc})";
            text = String.Format(text, quote.TotalRate);
            ShipMethodList.Items.Add(new ListItem(text, quote.ShipMethodId.ToString()));
        }
    }

    private int findSelectedIndex(int shipMethodId, ShipRateQuote[] arrRateQuotes)
    {
        for (int i = 0; i < arrRateQuotes.Length; i++)
        {
            ShipRateQuote quote = arrRateQuotes[i];
            if (quote.ShipMethodId == shipMethodId) return i;
        }

        // AS DEFAULT RETURN 0
        return 0;
    }

    private bool validateAndCreateAddresses()
    {
        //BILLING ADDRESS

        if (String.IsNullOrEmpty(BillToFirstName.Text) ||
            String.IsNullOrEmpty(BillToLastName.Text) ||
            String.IsNullOrEmpty(BillToAddress1.Text) ||
            String.IsNullOrEmpty(BillToCity.Text))
        {
            Page.Validate("BillingAddress");
            return false;
        }

        if (!ShipToBillingAddress.Checked)
        {
            if (String.IsNullOrEmpty(ShipToFirstName.Text) ||
            String.IsNullOrEmpty(ShipToLastName.Text) ||
            String.IsNullOrEmpty(ShipToAddress1.Text) ||
            String.IsNullOrEmpty(ShipToCity.Text))
            {
                Page.Validate("ShippingAddress");
                return false;
            }
        }

        // CREATE-UPDATE BILLING ADDRESS
        Address billAddress = _Basket.User.PrimaryAddress;
        if (_Basket.User.PrimaryAddress != null)
        {
            billAddress.Email = CustomerEmail.Text;
            billAddress.Address1 = BillToAddress1.Text;
            billAddress.Address2 = BillToAddress2.Text;
            billAddress.City = BillToCity.Text;
            billAddress.Company = BillToCompany.Text;
            billAddress.CountryCode = BillToCountryCode.Items[BillToCountryCode.SelectedIndex].Value;
            billAddress.FirstName = BillToFirstName.Text;
            billAddress.LastName = BillToLastName.Text;
            billAddress.Phone = BillToPhone.Text;
            billAddress.PostalCode = BillToPostalCode.Text;
            BillToProvince.Text = BillToProvince.Text.Trim().ToUpperInvariant();
            billAddress.Province = BillToProvince.Text;
            billAddress.Residence = (BillToResidence.SelectedIndex == 0);
            billAddress.Save();
        }

        // CREATE-UPDATE SHIPPING ADDRESS
        Address shipAddress;
        if (ShipToBillingAddress.Checked)
        {
            shipAddress = billAddress;
        }
        else
        {
            int sessionShipId = AlwaysConvert.ToInt(Session["CC0_SHIPTOADDRESSID"]);
            if (sessionShipId == 0 || sessionShipId == billAddress.AddressId)
            {
                // THE SESSION VALUE IS NOT PRESENT OR NO LONGER VALID
                shipAddress = new Address();
            }
            else
            {
                //IF THERE IS A VALUE ALREADY IN SESSION THEN UPDATE IT
                shipAddress = AddressDataSource.Load(sessionShipId);
            }
            shipAddress.UserId = _Basket.UserId;
            shipAddress.Address1 = ShipToAddress1.Text;
            shipAddress.Address2 = ShipToAddress2.Text;
            shipAddress.City = ShipToCity.Text;
            shipAddress.Company = ShipToCompany.Text;
            shipAddress.CountryCode = ShipToCountryCode.Items[ShipToCountryCode.SelectedIndex].Value;
            shipAddress.FirstName = ShipToFirstName.Text;
            shipAddress.LastName = ShipToLastName.Text;
            shipAddress.Phone = ShipToPhone.Text;
            shipAddress.PostalCode = ShipToPostalCode.Text;
            ShipToProvince.Text = ShipToProvince.Text.Trim().ToUpperInvariant();
            shipAddress.Province = ShipToProvince.Text;
            shipAddress.Residence = (ShipToResidence.SelectedIndex == 0);
            shipAddress.Save();

        }
        // STORE THE CUSTOMER SHIPPING ADDRESS ID IN SESSION        
        Session["CC0_SHIPTOADDRESSID"] = shipAddress.AddressId;
        return true;
    }

    protected void PlaceOrderButton_Click(object sender, EventArgs e)
    {
        if (Page.IsValid)
        {
            //PROCESS THE CHECKOUT
            _Basket.Recalculate();
            Payment payment = new Payment();
            payment.Amount = _Basket.Items.TotalPrice();
            CheckoutRequest checkoutRequest = new CheckoutRequest(null);
            CheckoutResponse checkoutResponse = _Basket.Checkout(checkoutRequest);
            if (checkoutResponse.Success)
            {
                Session.Remove("PlaceOrderUserId");
                Response.Redirect( "ViewOrder.aspx?OrderNumber=" + checkoutResponse.OrderNumber.ToString() + "&OrderId=" + checkoutResponse.OrderId.ToString());
            }
        }
    }

    protected void CustomerLookupButton_Click(object sender, EventArgs e)
    {
        int userId = UserDataSource.GetUserIdByEmail(CustomerEmail.Text, true);
        if (userId > 0)
        {
            //_Basket.UserId = userId;
            User user = UserDataSource.Load(userId);
            Basket.Transfer(_Basket.UserId, userId);
            _Basket.Package();
            _Basket.Recalculate();
            Response.Redirect("PlaceOrder2.aspx?BasketId=" + user.Basket.BasketId);
        }
    }

    protected void ApplyCouponButton_Click(object sender, EventArgs e)
    {
        UpdateShipMethod();
        ValidCouponMessage.Visible = false;
        Coupon coupon = CouponDataSource.LoadForCouponCode(CouponCode.Text);
        String couponValidityMessage = String.Empty;
        if (coupon != null)
        {
            if (!CouponCalculator.IsCouponAlreadyUsed(_Basket, coupon))
            {
                if (CouponCalculator.IsCouponValid(_Basket, coupon, out couponValidityMessage))
                {
                    BasketCoupon recentCoupon = new BasketCoupon(_Basket.UserId, coupon.CouponId);
                    _Basket.BasketCoupons.Add(recentCoupon);
                    // APPLY COUPON COMBINE RULE
                    //THE RULE: 
                    //If most recently applied coupon is marked "do not combine", then all previously
                    //entered coupons must be removed from Basket.

                    //If most recently applied coupon is marked "combine", then remove any applied
                    //coupon that is marked "do not combine".  (Logically, in this instance there
                    //could be at most one other coupon of this type...)
                    string previousCouponsRemoved = "";

                    if (recentCoupon.Coupon.AllowCombine)
                    {
                        //IF ALLOW COMBINE, REMOVE ALL PREVIOUS NOCOMBINE COUPONS
                        for (int i = (_Basket.BasketCoupons.Count - 1); i >= 0; i--)
                        {
                            if (!_Basket.BasketCoupons[i].Coupon.AllowCombine)
                            {
                                if (previousCouponsRemoved.Length > 0)
                                {
                                    previousCouponsRemoved += "," + _Basket.BasketCoupons[i].Coupon.Name;
                                }
                                else
                                {
                                    previousCouponsRemoved = _Basket.BasketCoupons[i].Coupon.Name;
                                }

                                _Basket.BasketCoupons.DeleteAt(i);
                            }
                        }
                    }
                    else
                    {
                        //IF NOT ALLOW COMBINE, REMOVE ALL EXCEPT THIS COUPON
                        for (int i = (_Basket.BasketCoupons.Count - 1); i >= 0; i--)
                        {
                            if (_Basket.BasketCoupons[i] != recentCoupon)
                            {
                                if (previousCouponsRemoved.Length > 0)
                                {
                                    previousCouponsRemoved += "," + _Basket.BasketCoupons[i].Coupon.Name;
                                }
                                else
                                {
                                    previousCouponsRemoved = _Basket.BasketCoupons[i].Coupon.Name;
                                }
                                _Basket.BasketCoupons.DeleteAt(i);
                            }
                        }
                    }

                    _Basket.Save();
                    _Basket.Recalculate();
                    if (previousCouponsRemoved.Length > 0)
                    {
                        if (recentCoupon.Coupon.AllowCombine)
                        {
                            CombineCouponRemoveMessage.Text = String.Format(CombineCouponRemoveMessage.Text, recentCoupon.Coupon.Name, previousCouponsRemoved, previousCouponsRemoved);
                            CombineCouponRemoveMessage.Visible = true;
                        }
                        else
                        {
                            NotCombineCouponRemoveMessage.Text = String.Format(NotCombineCouponRemoveMessage.Text, recentCoupon.Coupon.Name, previousCouponsRemoved);
                            NotCombineCouponRemoveMessage.Visible = true;
                        }
                    }
                    ValidCouponMessage.Visible = true;
                    BasketSummaryGrid.DataBind();
                }
                else
                {
                    InvalidCouponMessage.Text = couponValidityMessage;
                }
            }
            else
            {
                InvalidCouponMessage.Text = "The coupon code you've entered is already in use.";
            }
        }
        else
        {
            InvalidCouponMessage.Text = "The coupon code you've entered is invalid.";
        }
        CouponCode.Text = string.Empty;
        InvalidCouponMessage.Visible = !ValidCouponMessage.Visible;
    }

    protected void ShipMethodList_SelectedIndexChanged(object sender, EventArgs e)
    {
        UpdateShipMethod();
    }

    protected void ShipmentList_ItemDataBound(object sender, RepeaterItemEventArgs e)
    {
        //SHOW SHIPPING METHODS
        DropDownList ShipMethodList = (DropDownList)e.Item.FindControl("ShipMethodList");
        if (ShipMethodList != null)
        {
            BasketShipment shipment = (BasketShipment)e.Item.DataItem;
            if (shipment != null)
            {
                ICollection<ShipRateQuote> quotes = ShipRateQuoteDataSource.QuoteForShipment(shipment);
                BindShipMethodRateQuotes(ShipMethodList, quotes);

                if (quotes.Count == 0)
                {
                    //NO RATES AVAILABLE, WE HAVE TO DISABLE CHECKOUT
                    ShipMethodList.Visible = false;
                    _ShipMethodError = true;
                }
                else
                {
                    //MAKE SURE WE HAVE THE CORRECT ITEM SELECTED
                    int shipMethodId = AlwaysConvert.ToInt(Request.Form[ShipMethodList.UniqueID]);
                    ListItem selected = ShipMethodList.Items.FindByValue(shipMethodId.ToString());
                    if (selected != null) selected.Selected = true;
                }
            }
        }
    }

    protected int GetShipmentNumber(object bi){
        BasketItem basketItem = (BasketItem)bi;
        return _Basket.Shipments.IndexOf(basketItem.BasketShipmentId) + 1 ;
    }
}
